//
// Blister Ball - Ancient Apple ][ game
//
// (C)2000
// Brian Postma
// b.postma@hetnet.nl
//


#include "math.h"
#include "BlisterBall.h"

BlisterBall::BlisterBall()
{
  ingame=false;
  GameInit();
  GpSetPaletteEntry(0,  16, 24, 64);
  GpSetPaletteEntry(1,  255,255,255);
  GpSetPaletteEntry(2,  255,192,128);
  GpSetPaletteEntry(3,  255,0,0);
  GpSetPaletteEntry(4,  128,128,255);
  GpSetPaletteEntry(5,  96,128,255);

}


BlisterBall::~BlisterBall()
{
}


void BlisterBall::GameInit()
{ 
    int i;

    for (i=0; i<NUMBALLS; i++)
    {
      ballpos[i]=(int)(Random()*(float)(bounceperiod));
      ballx[i]=borderwidth*2+(int)(Random()*(float)(GetScreenWidth()-ballsize-4*borderwidth));
      ballampl[i]=storeheight-ballsize;
      bally[i]=storeheight+borderwidth-ballsize-
			(int)((float)ballampl[i]*sinf(3.141592*ballpos[i]/bounceperiod));
      if (Random()>=0.5)
        balldx[i]=1;
      else
        balldx[i]=-1;
      ballinstore[i]=true;
    }
    player1score=0;
    player1lives=3;
    player1fire=false;
    player1dpos=0;
    player1y=GetScreenHeight()-scoreheight-borderwidth*2-gutterheight-1;
    player1pos=0+borderwidth+32;
    bullet1xpos=-1;
    bullet1ypos=-1;
    ballsinplay=0;
    balldelta=6;
}


void BlisterBall::KeyDown(int key)
{
    if (ingame)
    {
      if (key == GPC_VK_LEFT)
        player1dpos=-3;
      if (key == GPC_VK_RIGHT)
        player1dpos=3;
      if (key == GPC_VK_FA )
        player1fire=true;
      if (key == GPC_VK_SELECT)
        ingame=false;
    }
    else
    {
      if (key == GPC_VK_START)
      {
        GameInit();
        ingame=true;
      }
    }
}

void BlisterBall::KeyUp(int key)
{
    if (key == GPC_VK_LEFT || key == GPC_VK_RIGHT)
       player1dpos=0;
}


void BlisterBall::Paint()
{
    ClrScreen();
    if (ingame)
      PlayGame();
    else
      ShowIntroScreen();
}


void BlisterBall::PlayGame()
{
    DrawPlayField();
    HandleMovement();
    DrawTurrets();
    CheckBallsInPlay();
    DoBalls();
    DoBullets();
    ShowScore();

    if (player1lives<=0 && player1pos>=GetScreenWidth())
      ingame=false;
}


void BlisterBall::HandleMovement()
{
    if (player1y<GetScreenHeight()-scoreheight-borderwidth*2-1)
    {
      player1pos+=player1dpos;
    }
    if (player1pos<=borderwidth)
      player1pos=borderwidth;
    if (player1pos>=(GetScreenWidth()-borderwidth-turretwidth-1) && player1lives>0)
      player1pos=GetScreenWidth()-borderwidth-turretwidth-1;
    if (player1fire && bullet1xpos<0 && bullet1ypos<0)
    {
      bullet1ypos=GetScreenHeight()-scoreheight-gutterheight-ballsize*2-borderwidth*2;
      bullet1xpos=player1pos+turretwidth/2;
    }
    player1fire=false;
}

 
void BlisterBall::CheckBallsInPlay()
{
    bool none=true;
    int i;

    for (i=0; i<NUMBALLS; i++)
    {
      if (!ballinstore[i])
        none=false;
    }
    if (none)
    {
      if (ballsinplay<NUMBALLS)
        ballsinplay++;
      else
         if (balldelta<=maxballdelta)
           balldelta+=4;

      for (i=0; i<ballsinplay; i++)
      {
        ballinstore[i]=false;
        ballampl[i]=GetScreenHeight()-4*borderwidth-scoreheight-storeheight-ballsize-gutterheight;
        ballpos[i]=bounceperiod/2;
      }
    }
}

void BlisterBall::ResetBall(int i)
{
    ballinstore[i]=true;
    ballampl[i]=storeheight-ballsize;
    ballx[i]=borderwidth*2+(int)(Random()*(float)(GetScreenWidth()-ballsize-4*borderwidth));
    bally[i]=storeheight+borderwidth-ballsize-
		(int)((float)ballampl[i]*sinf(3.141592*ballpos[i]/bounceperiod));
}


bool BlisterBall::DrawBullet(int x, int y)
{
    int i;

    DrawLine(x,y,x,y+ballsize,1);

    for (i=0; i<NUMBALLS; i++)
    {
      if ((bally[i]+ballsize)>=y && bally[i]<=(y+ballsize) &&
          (ballx[i]+ballsize)>=x && ballx[i]<=x)
      {
        ResetBall(i);
        return true;
      }
    }
    return false;
}


void BlisterBall::DoBullets()
{
    if (bullet1xpos>0 && bullet1ypos>0)
    {
      if (DrawBullet(bullet1xpos,bullet1ypos))
      {
        player1score+=10;
        bullet1xpos=-1;
        bullet1ypos=-1;
      }
      else
      {
        bullet1ypos-=bulletspeed;
        if (bullet1ypos<=(2*borderwidth+storeheight))
        {
          bullet1ypos=-1;
          bullet1xpos=-1;
        }
      }
    }
}

bool BlisterBall::DrawTurret(int x, int y)
{
    int x1=x;
    int y1=y;
    int x2=turretwidth+x;
    bool hitball=false;
    int i;

    while (x1<x2)
    {
      for (i=0; i<NUMBALLS; i++)
      {
        if ((bally[i]+ballsize>=y1) && bally[i]<=y1 && (ballx[i]+ballsize)>=x1 && ballx[i]<=x2)
        {
          hitball=true;
          ResetBall(i);
        }
      }
     
      DrawLine(x1,y1,x2,y1,2);
      y1--;
      x1++;
      x2--;
    }
    return hitball;
}    

 
void BlisterBall::DrawBall(int x, int y)
{
    FillRect(x,y+1,ballsize,ballsize-2,3);
    DrawLine(x+1,y,x+ballsize-2,y,3);
    DrawLine(x+1,y+ballsize-1,x+ballsize-2,y+ballsize-1,3);
}


void BlisterBall::DoBalls()
{
    int i;

    for (i=0; i<NUMBALLS; i++)
    {
      if (ballinstore[i])
      {
        bally[i]=storeheight+borderwidth-ballsize-
			(int)((float)ballampl[i]*sinf(3.141592*(float)ballpos[i]/bounceperiod));
        ballpos[i]=(ballpos[i]+1)%bounceperiod;
        ballx[i]=ballx[i]+balldx[i];
        if (ballx[i]<=borderwidth || ballx[i]>=(GetScreenWidth()-borderwidth-ballsize))
          balldx[i]=-balldx[i];
        DrawBall(ballx[i],bally[i]);
      }
      else
      {
        if (bally[i]<borderwidth*2+storeheight)
          bally[i]+=2;
        else
        {
          bally[i]=GetScreenHeight()-gutterheight-2*borderwidth-scoreheight-ballsize-
			(int)((float)ballampl[i]*sinf(3.141592*ballpos[i]/bounceperiod));
          ballpos[i]=(ballpos[i]+1)%bounceperiod;
          if (ballpos[i]==0)
          {
            ballampl[i]-=balldelta;
            if (ballampl[i]<minampl)
              ballampl[i]=minampl;
          }
        }
        ballx[i]=ballx[i]+balldx[i];
        if (ballx[i]<=borderwidth || ballx[i]>=(GetScreenWidth()-borderwidth-ballsize))
          balldx[i]=-balldx[i];
        DrawBall(ballx[i],bally[i]);
      }
    }
}


void BlisterBall::ShowIntroScreen()
{
    DrawPlayField();
    ShowScore();
    DoBalls();

    char s1[]="Blister Ball";
    char s2[]="An ancient Apple ][ game";
    char s3[]="(c)2000,2003 by Brian Postma";
    char s4[]="B.Postma@HetNet.nl";
    char s5[]="http://www.brianpostma.com";

    DrawString(s1,(GetScreenWidth()-8*strlen(s1))/2,80,5);
    DrawString(s2,(GetScreenWidth()-8*strlen(s2))/2,105,1);
    DrawString(s3,(GetScreenWidth()-8*strlen(s3))/2,120,1);
    DrawString(s4,(GetScreenWidth()-8*strlen(s4))/2,135,2);
    DrawString(s5,(GetScreenWidth()-8*strlen(s5))/2,150,2);
}


void BlisterBall::DrawTurrets()
{
    int delta;
    if (player1lives==0)
    {
      if (player1y<GetScreenHeight()-scoreheight-borderwidth*2-1)
        player1y++;
      else if (player1pos<GetScreenWidth()+turretwidth)
        player1pos++;
    }
    if (DrawTurret(player1pos,player1y))
    {
      player1lives--;
      player1y+=2;
    }
}


void BlisterBall::DrawPlayField()
{
    FillRect(0,0,GetScreenWidth(),borderwidth,1);
    FillRect(0,0,borderwidth,GetScreenHeight()-scoreheight,1);
    FillRect(GetScreenWidth()-borderwidth,0,borderwidth,GetScreenHeight()-scoreheight,1);
    FillRect(0,0+borderwidth+storeheight,GetScreenWidth(),borderwidth,1);
    FillRect(0,GetScreenHeight()-gutterheight-2*borderwidth-scoreheight,GetScreenWidth(),borderwidth,1);
    FillRect(0,GetScreenHeight()-scoreheight-borderwidth,GetScreenWidth(),borderwidth,1);
    FillRect(borderwidth,GetScreenHeight()-borderwidth-gutterheight-scoreheight,
			GetScreenWidth()-borderwidth,gutterheight,4);
}


void BlisterBall::ShowScore()
{
    char buf[24];

    sprintf(buf,"Score: %05d",player1score);
    DrawString(buf,borderwidth,GetScreenHeight()-16,1);
}

